package hal

import (
	"context"
	"golang.org/x/sync/errgroup"
)

type Cluster struct {
	modules []*Module
}

func (c *Cluster) Run(ctx context.Context) error {
	errs, _ := errgroup.WithContext(ctx)
	for _, module := range c.modules {
		errs.Go(module.Run)
	}
	return errs.Wait()
}

type ModuleMapping map[MultiThreadedIOPort]MultiThreadedIOPort

func (mapping ModuleMapping) BuildCluster() *Cluster {
	modules := make(map[*Module]struct{})
	for host, client := range mapping {
		modules[host.Module] = struct{}{}
		modules[client.Module] = struct{}{}
		host.Module.IO[host.IOPort] = NewMultiThreadedIO(&client)
	}
	cluster := &Cluster{modules: make([]*Module, len(modules), len(modules))}
	var index int64
	for module := range modules {
		cluster.modules[index] = module
		index++
	}
	return cluster
}
